@@ -10,11 +10,11 @@ Lugh - Pure C LLM Inference Engine for Perl (built on ggml)
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 require XSLoader;
 XSLoader::load('Lugh', $VERSION);
@@ -270,6 +270,76 @@ Feed-forward network with gated activation:
 
 =head1 PERFORMANCE
 
+=head2 Backend Selection
+
+Lugh supports multiple compute backends through the ggml library:
+
+    # List available backends on your system
+    my @backends = Lugh::available_backends();
+    # Returns: ('Metal', 'BLAS', 'CPU', 'auto')
+    
+    # Check which backend is best for GPU acceleration
+    my $best = Lugh::best_backend();
+    # Returns: 'Metal' on macOS with GPU, 'CPU' otherwise
+    
+    # Get detailed info about a backend
+    my $info = Lugh::backend_info('Metal');
+    # Returns: { name => 'Metal', type => 'GPU', is_gpu => 1, 
+    #            description => 'Apple M1', device_count => 1 }
+    
+    # Check if a specific backend is available
+    if (Lugh::backend_available('Metal')) {
+        print "Metal GPU acceleration available!\n";
+    }
+    
+    # Select backend for inference
+    my $inference = Lugh::Inference->new(
+        model   => $model,
+        backend => 'auto',     # auto-select best (default)
+        # backend => 'Metal',  # force Metal GPU
+        # backend => 'CPU',    # force CPU
+    );
+
+Available backends depend on your system and ggml installation:
+
+=over 4
+
+=item * B<Metal> - Apple GPU (macOS only)
+
+=item * B<CUDA> - NVIDIA GPU (requires ggml with CUDA)
+
+=item * B<Vulkan> - Cross-platform GPU
+
+=item * B<BLAS> - Accelerate/OpenBLAS for fast matrix ops
+
+=item * B<CPU> - Always available, uses SIMD
+
+=item * B<auto> - Automatically select the best available
+
+=back
+
+=head2 Backend API Functions
+
+=over 4
+
+=item * C<available_backends()> - List all available backend names
+
+=item * C<backend_count()> - Number of registered backends
+
+=item * C<backend_device_count()> - Total number of available devices
+
+=item * C<backend_info($name)> - Get detailed info about a backend
+
+=item * C<backend_available($name)> - Check if a backend is available
+
+=item * C<best_backend()> - Get the name of the best available backend
+
+=item * C<has_metal()> - Check if Metal is compiled in (macOS)
+
+=item * C<metal_available()> - Check if Metal GPU is available at runtime
+
+=back
+
 =head2 Memory Usage
 
 Memory depends on model size and quantization:
@@ -313,10 +383,6 @@ External resources:
 
 =item * L<https://github.com/ggerganov/ggml> - The ggml tensor library
 
-=item * L<https://github.com/ggerganov/llama.cpp> - Reference implementation
-
-=item * L<https://arxiv.org/abs/1706.03762> - "Attention Is All You Need"
-
 =back
 
 =head1 AUTHOR
