@@ -13,7 +13,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -120,7 +120,9 @@ used by LLaMA, Mistral, and similar models:
 
     my $inference = Lugh::Inference->new(
         model      => $model,
-        flash_attn => 0,  # optional, use flash attention
+        backend    => 'auto',  # optional, compute backend
+        n_threads  => 4,       # optional, number of CPU threads
+        flash_attn => 0,       # optional, use flash attention
     );
 
 Creates a new Inference engine from a loaded model.
@@ -131,23 +133,55 @@ B<Parameters:>
 
 =item * C<model> (required) - A Lugh::Model object
 
+=item * C<backend> (optional) - Compute backend to use. Defaults to 'auto'.
+
+Available backends depend on your system:
+
+=over 8
+
+=item * C<'auto'> - Automatically select the best available (GPU preferred)
+
+=item * C<'Metal'> - Apple Metal GPU (macOS only)
+
+=item * C<'CUDA'> - NVIDIA GPU (if ggml built with CUDA)
+
+=item * C<'Vulkan'> - Cross-platform GPU
+
+=item * C<'CPU'> - CPU with SIMD (always available)
+
+=back
+
+Use C<Lugh::available_backends()> to see what's available on your system.
+
+=item * C<n_threads> (optional) - Number of CPU threads for computation.
+Defaults to 4. Only affects CPU backend.
+
 =item * C<flash_attn> (optional) - Use flash attention if set to 1 (default: 0)
 
 =back
 
 B<Returns:> A Lugh::Inference object.
 
-B<Throws:> Dies if no model is provided.
+B<Throws:> Dies if no model is provided or if requested backend is unavailable.
 
 B<Example:>
 
     my $model = Lugh::Model->new(model => 'model.gguf');
+    
+    # Auto-select best backend (recommended)
     my $inference = Lugh::Inference->new(model => $model);
     
-    # Or with flash attention enabled
-    my $fast_inference = Lugh::Inference->new(
-        model      => $model,
-        flash_attn => 1,
+    # Force Metal GPU on macOS
+    my $gpu_inference = Lugh::Inference->new(
+        model   => $model,
+        backend => 'Metal',
+    );
+    
+    # Force CPU with 8 threads
+    my $cpu_inference = Lugh::Inference->new(
+        model     => $model,
+        backend   => 'CPU',
+        n_threads => 8,
     );
 
 =head1 METHODS
@@ -614,20 +648,126 @@ Current implementation:
 
 =item * Quantized weights stay quantized during computation
 
+=item * KV cache for incremental decoding (see C<create_kv_cache>)
+
+=item * Memory pools for efficient repeated inference
+
+=item * Batch processing for multiple sequences
+
 =back
 
-Potential improvements:
+=head1 ADVANCED METHODS
+
+=head2 create_memory_pool
+
+    my $pool = $inference->create_memory_pool();
+
+Creates a reusable memory pool for efficient repeated forward passes.
+The pool caches backend and allocator resources, avoiding per-call
+allocation overhead.
+
+B<Returns:> A Lugh::MemoryPool object.
+
+B<Example:>
+
+    my $pool = $inference->create_memory_pool();
+    
+    # Efficient repeated forward passes
+    for my $text (@texts) {
+        my @tokens = $tokenizer->encode($text);
+        my @logits = $inference->forward_with_pool($pool, \@tokens);
+        # Process logits...
+    }
+    
+    # Pool automatically cleaned up on destruction
+    # Or manually reset for next batch:
+    $pool->reset();
+
+=head2 forward_with_pool
+
+    my @logits = $inference->forward_with_pool($pool, \@tokens);
+
+Runs forward pass using a pre-allocated memory pool. More efficient
+than C<forward()> for repeated inference on different inputs.
+
+B<Parameters:>
 
 =over 4
 
-=item * KV cache for incremental decoding
+=item * C<$pool> - A Lugh::MemoryPool from C<create_memory_pool()>
 
-=item * Batch processing multiple sequences
-
-=item * Flash attention for long contexts
+=item * C<\@tokens> - Array reference of token IDs
 
 =back
 
+B<Returns:> A list of logits (one per vocabulary token).
+
+B<Example:>
+
+    my $pool = $inference->create_memory_pool();
+    
+    # Much more efficient than calling forward() repeatedly
+    for my $prompt (@prompts) {
+        my @tokens = $tokenizer->encode($prompt);
+        my @logits = $inference->forward_with_pool($pool, \@tokens);
+        my $next_token = $inference->sample_top_p(\@logits, top_p => 0.9);
+        print $tokenizer->decode([$next_token]), "\n";
+    }
+
+=head2 forward_batch
+
+    my $results = $inference->forward_batch(\@sequences);
+
+Processes multiple token sequences, returning logits for each.
+Each sequence is processed independently with shared backend resources.
+
+B<Parameters:>
+
+=over 4
+
+=item * C<\@sequences> - Array reference of array references of token IDs
+
+=back
+
+B<Returns:> Array reference of array references of logits.
+
+B<Example:>
+
+    my @seq1 = $tokenizer->encode("Hello");
+    my @seq2 = $tokenizer->encode("World");
+    my @seq3 = $tokenizer->encode("Test");
+    
+    my $results = $inference->forward_batch([\@seq1, \@seq2, \@seq3]);
+    
+    # $results->[0] is logits for seq1
+    # $results->[1] is logits for seq2
+    # $results->[2] is logits for seq3
+    
+    for my $i (0 .. $#$results) {
+        my @logits = @{$results->[$i]};
+        my $next = $inference->sample_top_p(\@logits, top_p => 0.9);
+        print "Sequence $i next token: ", $tokenizer->decode([$next]), "\n";
+    }
+
+=head1 MEMORY POOL
+
+The Lugh::MemoryPool class provides reusable compute resources:
+
+=head2 reset
+
+    $pool->reset();
+
+Resets the memory pool for reuse. Frees and reallocates the
+compute context. Called automatically by C<forward_with_pool()>.
+
+B<Returns:> True on success.
+
+=head2 backend
+
+    my $backend_name = $pool->backend();
+
+Returns the name of the backend used by this pool (e.g., "Metal", "CPU").
+
 =head1 THREAD SAFETY
 
 Lugh::Inference objects are NOT thread-safe. Each Perl thread must
